#!perl.exe
#
############################################################
#Christopher Anderson's Time Calculation Library for Perl 5#
#http://www.techstorm.net/perl/							   #
############################################################
#
#time array must be in this form for accurate results:
#	@time = [$second, $minute, $hour, $date, $month, $year, $weekDay, $dayOfYear, $isDST];
#
#Notes:
#
#-	$date is a value between 1 and the last day of the month
#
#-	$month is a value between 0 and 11 with 0 representing January
#
#-	$weekDay is a value between 0 and 6 with 0 representing Sunday
#
#-	if you use the advanceMonth or advanceYear functions, $weekDay will not be calculated and
#	will not be accurate
#
#-	$dayOfYear and $isDST are not calculated and will not be accurate, but are included to be 
#	compatible with the output from localtime(time) (ex. for the current time use 
#   "@time = localtime(time)")
#
#Functions Available:
#
#	advanceSec()
#	advanceMin()
#	advanceHour()
#	advanceDate()
#	advanceMonth()
#	advanceYear()
#	reverseSec()
#	reverseMin()
#	reverseHour()
#	reverseDate()
#	reverseMonth()
#	reverseYear()
#	compare()
#
#Advance/Reverse Example:
#
#	@newtime = advanceSec(\@time, $interval);

#compare() Examples
#
#-	sub returns true if @time1 is greater than or equal to @time2, otherwise it returns false
#-	sub returns 2 (which also evaluates as true) if times are equal
#
#-	greater than or equal to (>=)
#	if (&compare(\@time1, \@time2)) { do this if @time1 is greater than or equal to @time2 }
#
#-	is less than (<)
#	if (!(&compare(\@time1, \@time2))) { do this if @time1 is less than @time2 }
#
#-	is equal to (==)
#	if (&compare(\@time1, \@time2) == 2) { do this if @time1 is equal to @time2 }
#
#-	is less than or equal to (<=)
#	if (!(&compare(\@time1, \@time2)) || &compare(\@time1, \@time2) == 2) { do this if @time1 is less than or equal to @time2 }

sub advanceSec
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[0] += $interval;
	while ($time[0] >= 60)
	{
		@time = &advanceMin(\@time, 1);
		$time[0] -= 60;
	}
	
	return @time;
}

sub advanceMin
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[1] += $interval;
	while ($time[1] >= 60)
	{
		@time = &advanceHour(\@time, 1);
		$time[1] -= 60;
	}
	
	return @time;
}

sub advanceHour
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[2] += $interval;
	while ($time[2] >= 24)
	{
		@time = &advanceDate(\@time, 1);
		$time[2] -= 24;
	}
	
	return @time;
}

sub advanceDate
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	my @daysPerMonth = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
	
	$time[3] += $interval;
	while ($time[3] > $daysPerMonth[$time[4]])
	{
		if ($time[4] == 1 && ($time[5] % 400 == 0 || ($time[5] % 4 == 0 && $time[5] % 100 != 0)))
		{
			if ($time[3] > 29)
			{
				@time = &advanceMonth(\@time, 1);
				$time[3] -= 29;
			}
			else
			{
				last;
			}
		}
		else
		{
			$time[3] -= $daysPerMonth[$time[4]];
			@time = &advanceMonth(\@time, 1);
		}
	}

	#calculate week day
	for (my $a = 0; $a < $interval; $a++)
	{
		$time[6]++;
		
		if ($time[6] > 6)
		{
			$time[6] = 0;
		}
	}
	
	return @time;
}

sub advanceMonth
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[4] += $interval;
	while ($time[4] > 11)
	{
		@time = &advanceYear(\@time, 1);
		$time[4] -= 12;
	}
	@time = &advanceDate(\@time, 0);
	
	return @time;
}

sub advanceYear
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[5] += $interval;
	
	return @time;
}

sub reverseSec
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[0] -= $interval;
	while ($time[0] < 0)
	{
		@time = &reverseMin(\@time, 1);
		$time[0] += 60;
	}
	
	return @time;
}

sub reverseMin
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[1] -= $interval;
	while ($time[1] < 0)
	{
		@time = &reverseHour(\@time, 1);
		$time[1] += 60;
	}
	
	return @time;
}

sub reverseHour
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[2] -= $interval;
	while ($time[2] < 0)
	{
		@time = &reverseDate(\@time, 1);
		$time[2] += 24;
	}
	
	return @time;
}

sub reverseDate
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	my @daysPerMonth = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
	
	$time[3] -= $interval;
	while ($time[3] < 1)
	{
		if ($time[4] == 2 && ($time[5] % 400 == 0 || ($time[5] % 4 == 0 && $time[5] % 100 != 0)))
		{
			$time[3] += 29;
		}
		else
		{
			$time[3] += $daysPerMonth[$time[4]-1];
		}

		@time = &reverseMonth(\@time, 1);
	}

	for (my $a = 0; $a < $interval; $a++)
	{
		$time[6]--;
		
		if ($time[6] < 0)
		{
			$time[6] = 6;
		}
	}
	
	return @time;
}

sub reverseMonth
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[4] -= $interval;
	while ($time[4] < 0)
	{
		@time = &reverseYear(\@time, 1);
		$time[4] += 12;
	}
	@time = &advanceDate(\@time, 0);
	
	return @time;
}

sub reverseYear
{
	(my $ptrTime, my $interval) = @_;
	my @time = @$ptrTime;
	
	$time[5] -= $interval;
	
	return @time;
}

sub compare
{
	(my $ptrTime1, my $ptrTime2) = @_;
	my @time1 = @$ptrTime1;
	my @time2 = @$ptrTime2;
	
	#perl doesn't do boolean values
	#true = not 0
	#false = 0
	
	if ($time1[5] > $time2[5])
	{
		return 1;
	}
	else
	{
		if ($time1[5] < $time2[5])
		{
			return 0;
		}
	}

	if ($time1[4] > $time2[4])
	{
		return 1;
	}
	else
	{
		if ($time1[4] < $time2[4])
		{
			return 0;
		}
	}
			
	if ($time1[3] > $time2[3])
	{
		return 1;
	}
	else
	{
		if ($time1[3] < $time2[3])
		{
			return 0;
		}
	}

	if ($time1[2] > $time2[2])
	{
		return 1;
	}
	else
	{
		if ($time1[2] < $time2[2])
		{
			return 0;
		}
	}

	if ($time1[1] > $time2[1])
	{
		return 1;
	}
	else
	{
		if ($time1[1] < $time2[1])
		{
			return 0;
		}
	}

	if ($time1[0] > $time2[0])
	{
		return 1;
	}
	
	if ($time1[0] == $time2[0])
	{
		return 2;
	}

	return 0;
}

return 1;